@@ -87,7 +87,19 @@ class InfoJob(Job):
         pass
 
     def execute(self):
-        self.storage.info()
+        info = self.storage.info()
+        if not info:
+            return
+        fields = ["Container", "Size", "Object Count"]
+        data = []
+        for container in info:
+            values = [
+                container.get('container_name'),
+                container.get('size'),
+                container.get('objects_count')
+            ]
+            data.append(values)
+        return [fields, data]
 
 
 class BackupJob(Job):
@@ -101,6 +113,22 @@ class BackupJob(Job):
                 raise Exception(
                     'no-incremental option is not compatible '
                     'with backup level options')
+        if self.conf.mode == 'nova':
+            if not self.conf.no_incremental:
+                raise ValueError("Incremental nova backup is not supported")
+
+            if not self.conf.nova_inst_id and not self.conf.project_id:
+                raise ValueError("nova-inst-id or project-id"
+                                 " argument must be provided")
+
+        if self.conf.mode == 'cinder':
+            if not self.conf.cinder_vol_id:
+                raise ValueError("cinder-vol-id argument must be provided")
+
+        if self.conf.mode == "cindernative":
+            if not self.conf.cindernative_vol_id:
+                raise ValueError("cindernative-vol-id"
+                                 " argument must be provided")
 
     def execute(self):
         LOG.info('Backup job started. '
@@ -195,7 +223,7 @@ class BackupJob(Job):
 
                 # Checksum for Backup Consistency
                 if self.conf.consistency_check:
-                    ignorelinks = (self.conf.dereference_symlink == 'none' or
+                    ignorelinks = (self.conf.dereference_symlink is None or
                                    self.conf.dereference_symlink == 'hard')
                     consistency_checksum = checksum.CheckSum(
                         filepath, ignorelinks=ignorelinks).compute()
@@ -204,7 +232,7 @@ class BackupJob(Job):
                     self.conf.consistency_checksum = consistency_checksum
 
                 return self.engine.backup(
-                    backup_path=filepath,
+                    backup_resource=filepath,
                     hostname_backup_name=self.conf.hostname_backup_name,
                     no_incremental=self.conf.no_incremental,
                     max_level=self.conf.max_level,
@@ -224,9 +252,28 @@ class BackupJob(Job):
                                     self.storage)
 
         if backup_media == 'nova':
-            LOG.info('Executing nova backup. Instance ID: {0}'.format(
-                self.conf.nova_inst_id))
-            backup_os.backup_nova(self.conf.nova_inst_id)
+            if self.conf.project_id:
+                return self.engine.backup_nova_tenant(
+                    project_id=self.conf.project_id,
+                    hostname_backup_name=self.conf.hostname_backup_name,
+                    no_incremental=self.conf.no_incremental,
+                    max_level=self.conf.max_level,
+                    always_level=self.conf.always_level,
+                    restart_always_level=self.conf.restart_always_level)
+            else:
+                LOG.info('Executing nova backup. Instance ID: {0}'.format(
+                    self.conf.nova_inst_id))
+                hostname_backup_name = os.path.join(
+                    self.conf.hostname_backup_name,
+                    self.conf.nova_inst_id)
+                return self.engine.backup(
+                    backup_resource=self.conf.nova_inst_id,
+                    hostname_backup_name=hostname_backup_name,
+                    no_incremental=self.conf.no_incremental,
+                    max_level=self.conf.max_level,
+                    always_level=self.conf.always_level,
+                    restart_always_level=self.conf.restart_always_level)
+
         elif backup_media == 'cindernative':
             LOG.info('Executing cinder native backup. Volume ID: {0}, '
                      'incremental: {1}'.format(self.conf.cindernative_vol_id,
@@ -238,6 +285,16 @@ class BackupJob(Job):
             LOG.info('Executing cinder snapshot. Volume ID: {0}'.format(
                 self.conf.cinder_vol_id))
             backup_os.backup_cinder_by_glance(self.conf.cinder_vol_id)
+        elif backup_media == 'cinderbrick':
+            LOG.info('Executing cinder volume backup using os-brick. '
+                     'Volume ID: {0}'.format(self.conf.cinderbrick_vol_id))
+            return self.engine.backup(
+                backup_resource=self.conf.cinderbrick_vol_id,
+                hostname_backup_name=self.conf.hostname_backup_name,
+                no_incremental=self.conf.no_incremental,
+                max_level=self.conf.max_level,
+                always_level=self.conf.always_level,
+                restart_always_level=self.conf.restart_always_level)
         else:
             raise Exception('unknown parameter backup_media %s' % backup_media)
         return None
@@ -246,9 +303,12 @@ class BackupJob(Job):
 class RestoreJob(Job):
 
     def _validate(self):
-        if not self.conf.restore_abs_path and not self.conf.nova_inst_id \
-                and not self.conf.cinder_vol_id and not \
-                self.conf.cindernative_vol_id:
+        if not any([self.conf.restore_abs_path,
+                    self.conf.nova_inst_id,
+                    self.conf.cinder_vol_id,
+                    self.conf.cindernative_vol_id,
+                    self.conf.cinderbrick_vol_id,
+                    self.conf.project_id]):
             raise ValueError("--restore-abs-path is required")
         if not self.conf.container:
             raise ValueError("--container is required")
@@ -269,9 +329,10 @@ class RestoreJob(Job):
         if conf.backup_media == 'fs':
             self.engine.restore(
                 hostname_backup_name=self.conf.hostname_backup_name,
-                restore_path=restore_abs_path,
+                restore_resource=restore_abs_path,
                 overwrite=conf.overwrite,
-                recent_to_date=restore_timestamp)
+                recent_to_date=restore_timestamp,
+                backup_media=conf.mode)
 
             try:
                 if conf.consistency_checksum:
@@ -293,12 +354,28 @@ class RestoreJob(Job):
         res = restore.RestoreOs(conf.client_manager, conf.container,
                                 self.storage)
         if conf.backup_media == 'nova':
-            LOG.info("Restoring nova backup. Instance ID: {0}, timestamp: {1} "
-                     "network-id {2}".format(conf.nova_inst_id,
-                                             restore_timestamp,
-                                             conf.nova_network_id))
-            res.restore_nova(conf.nova_inst_id, restore_timestamp,
-                             conf.nova_network_id)
+            if self.conf.project_id:
+                return self.engine.restore_nova_tenant(
+                    project_id=self.conf.project_id,
+                    hostname_backup_name=self.conf.hostname_backup_name,
+                    overwrite=conf.overwrite,
+                    recent_to_date=restore_timestamp)
+            else:
+                LOG.info("Restoring nova backup. Instance ID: {0}, "
+                         "timestamp: {1} network-id {2}".format(
+                             conf.nova_inst_id,
+                             restore_timestamp,
+                             conf.nova_restore_network))
+                hostname_backup_name = os.path.join(
+                    self.conf.hostname_backup_name,
+                    self.conf.nova_inst_id)
+                self.engine.restore(
+                    hostname_backup_name=hostname_backup_name,
+                    restore_resource=conf.nova_inst_id,
+                    overwrite=conf.overwrite,
+                    recent_to_date=restore_timestamp,
+                    backup_media=conf.mode)
+
         elif conf.backup_media == 'cinder':
             LOG.info("Restoring cinder backup from glance. Volume ID: {0}, "
                      "timestamp: {1}".format(conf.cinder_vol_id,
@@ -312,6 +389,16 @@ class RestoreJob(Job):
             res.restore_cinder(conf.cindernative_vol_id,
                                conf.cindernative_backup_id,
                                restore_timestamp)
+        elif conf.backup_media == 'cinderbrick':
+            LOG.info("Restoring cinder backup using os-brick. Volume ID {0}, "
+                     "timestamp: {1}".format(conf.cinderbrick_vol_id,
+                                             restore_timestamp))
+            self.engine.restore(
+                hostname_backup_name=self.conf.hostname_backup_name,
+                restore_resource=conf.cinderbrick_vol_id,
+                overwrite=conf.overwrite,
+                recent_to_date=restore_timestamp,
+                backup_media=conf.mode)
         else:
             raise Exception("unknown backup type: %s" % conf.backup_media)
         return {}
@@ -331,14 +418,75 @@ class AdminJob(Job):
             timestamp = utils.date_to_timestamp(self.conf.remove_from_date)
         else:
             timestamp = datetime.datetime.now() - \
-                datetime.timedelta(days=self.conf.remove_older_than)
+                datetime.timedelta(days=float(self.conf.remove_older_than))
             timestamp = int(time.mktime(timestamp.timetuple()))
 
-        self.storage.remove_older_than(self.engine,
-                                       timestamp,
-                                       self.conf.hostname_backup_name)
+        if self.conf.backup_media == 'cinder':
+            old_backups = self.get_cinder_old_backups(
+                timestamp,
+                self.conf.cinder_vol_id
+            )
+            self.remove_backup_dirs(old_backups, self.conf.cinder_vol_id)
+            return {}
+
+        hostname_backup_name_set = set()
+
+        if self.conf.backup_media == 'nova':
+            if self.conf.project_id:
+                instance_ids = self.engine.get_nova_tenant(
+                    self.conf.project_id)
+                for instance_id in instance_ids:
+                    hostname_backup_name = os.path.join(
+                        self.conf.hostname_backup_name, instance_id)
+                    hostname_backup_name_set.add(hostname_backup_name)
+            else:
+                hostname_backup_name = os.path.join(
+                    self.conf.hostname_backup_name,
+                    self.conf.nova_inst_id)
+                hostname_backup_name_set.add(hostname_backup_name)
+        else:
+            hostname_backup_name_set.add(self.conf.hostname_backup_name)
+
+        for backup_name in hostname_backup_name_set:
+            self.storage.remove_older_than(self.engine,
+                                           timestamp,
+                                           backup_name)
         return {}
 
+    def get_cinder_old_backups(self, timestamp, cinder_vol_id):
+        path_to_list = self.get_path_prefix(cinder_vol_id)
+
+        old_backups = []
+        backup_dirs = self.storage.listdir(path_to_list)
+        for backup_dir in backup_dirs:
+            if int(backup_dir) <= int(timestamp):
+                old_backups.append(backup_dir)
+
+        return old_backups
+
+    def remove_backup_dirs(self, backups_to_remove, cinder_vol_id):
+        path_prefix = self.get_path_prefix(cinder_vol_id)
+        for backup_to_remove in backups_to_remove:
+            path_to_remove = "{0}/{1}".format(path_prefix, backup_to_remove)
+            LOG.info("Remove backup: {0}".format(path_to_remove))
+            self.storage.rmtree(path_to_remove)
+
+    def get_path_prefix(self, cinder_vol_id):
+        if self.storage.type == 'swift':
+            path_prefix = "{0}/{1}/{2}".format(
+                self.storage.container,
+                self.storage.segments,
+                cinder_vol_id
+            )
+        elif self.storage.type in ['local', 'ssh', 's3']:
+            path_prefix = "{0}/{1}".format(
+                self.storage.storage_path,
+                cinder_vol_id
+            )
+        else:
+            path_prefix = ''
+        return path_prefix
+
 
 class ExecJob(Job):
 
