@@ -46,9 +46,18 @@ class RestoreOs(object):
         if self.storage.type == "swift":
             swift = self.client_manager.get_swift()
             path = "{0}_segments/{1}/".format(self.container, path)
-            info, backups = swift.get_container(self.container, prefix=path)
+            container_name, path = self.get_storage_info(path)
+            info, backups = swift.get_container(container_name, prefix=path)
             backups = sorted(
                 map(lambda x: int(x["name"].rsplit("/", 1)[-1]), backups))
+        elif self.storage.type == "s3":
+            bucket_name, path = self.get_storage_info(path)
+            backups = self.storage.list_all_objects(
+                bucket_name=bucket_name,
+                prefix=path
+            )
+            backups = sorted(
+                map(lambda x: int(x['Key'].split("/", 2)[1]), backups))
         elif self.storage.type == "local":
             path = "{0}/{1}".format(self.container, path)
             backups = os.listdir(os.path.abspath(path))
@@ -67,6 +76,14 @@ class RestoreOs(object):
             raise BaseException(msg)
         return backups[-1]
 
+    def get_storage_info(self, path):
+        storage_info = self.container.split('/', 1)
+        container_name = storage_info[0]
+        object_prefix = storage_info[1] if '/' in self.container else ''
+        if object_prefix != '':
+            path = "{0}/{1}".format(object_prefix, path)
+        return container_name, path
+
     def _create_image(self, path, restore_from_timestamp):
         """
         :param path:
@@ -74,9 +91,9 @@ class RestoreOs(object):
         :type restore_from_timestamp: int
         :return:
         """
-        swift = self.client_manager.get_swift()
         backup = self._get_backups(path, restore_from_timestamp)
         if self.storage.type == 'swift':
+            swift = self.client_manager.get_swift()
             path = "{0}_segments/{1}/{2}".format(self.container, path, backup)
             stream = swift.get_object(self.container,
                                       "{}/{}".format(path, backup),
@@ -84,6 +101,40 @@ class RestoreOs(object):
             length = int(stream[0]["x-object-meta-length"])
             data = utils.ReSizeStream(stream[1], length, 1)
             info = stream[0]
+            image = self.client_manager.create_image(
+                name="restore_{}".format(path),
+                container_format="bare",
+                disk_format="raw",
+                data=data)
+            return info, image
+        elif self.storage.type == 's3':
+            if self.storage.get_object_prefix() != '':
+                base_path = "{0}/{1}/{2}".format(
+                    self.storage.get_object_prefix(),
+                    path,
+                    backup
+                )
+            else:
+                base_path = "{0}/{1}".format(path, backup)
+            image_file = "{0}/{1}".format(base_path, path)
+            s3_object = self.storage.get_object(
+                bucket_name=self.storage.get_bucket_name(),
+                key=image_file
+            )
+            stream = utils.S3ResponseStream(data=s3_object['Body'],
+                                            chunk_size=10000000)
+            data = utils.ReSizeStream(
+                stream,
+                s3_object['ContentLength'],
+                1
+            )
+            metadata = "{0}/metadata".format(base_path)
+            metadata_object = self.storage.get_object(
+                bucket_name=self.storage.get_bucket_name(),
+                key=metadata
+            )
+            info = json.load(metadata_object['Body'])
+
             image = self.client_manager.create_image(
                 name="restore_{}".format(path),
                 container_format="bare",
@@ -139,7 +190,6 @@ class RestoreOs(object):
         :param restore_from_timestamp:
         :return:
         """
-        backup = None
         cinder = self.client_manager.get_cinder()
         search_opts = {
             'volume_id': volume_id,
@@ -208,49 +258,3 @@ class RestoreOs(object):
 
         LOG.info("Deleting temporary image {}".format(image.id))
         self.client_manager.get_glance().images.delete(image.id)
-
-    def restore_nova(self, instance_id, restore_from_timestamp,
-                     nova_network=None):
-        """
-        :param restore_from_timestamp:
-        :type restore_from_timestamp: int
-        :param instance_id: id of attached nova instance
-        :param nova_network: id of network
-        :return:
-        """
-        # TODO(yangyapeng): remove nova_network check use nova api,
-        # nova api list network is not accurate.
-        # Change validation use neutron api instead of nova api in
-        # a project, find all available network in restore nova.
-        # implementation it after tenant backup add get_neutron in
-        # openstack oslient.
-        nova = self.client_manager.get_nova()
-        (info, image) = self._create_image(instance_id, restore_from_timestamp)
-        flavor = nova.flavors.get(info['x-object-meta-flavor-id'])
-        LOG.info("Creating an instance")
-        instance = None
-        if nova_network:
-            nics_id = [nic.id for nic in nova.networks.findall()]
-            if nova_network not in nics_id:
-                raise Exception("The network %s is invalid" % nova_network)
-            instance = nova.servers.create(info['x-object-meta-name'],
-                                           image, flavor,
-                                           nics=[{'net-id': nova_network}])
-        else:
-            try:
-                instance = nova.servers.create(info['x-object-meta-name'],
-                                               image, flavor)
-            except Exception as e:
-                LOG.warn(e)
-                raise Exception("The parameter --nova-restore-network "
-                                "is required")
-        # loop and wait till the server is up then remove the image
-        # let's wait 100 second
-        LOG.info('Delete instance image from glance {0}'.format(image))
-        for i in range(0, 360):
-            time.sleep(10)
-            instance = nova.servers.get(instance)
-            if not instance.__dict__['OS-EXT-STS:task_state']:
-                glance = self.client_manager.create_glance()
-                glance.images.delete(image.id)
-                return
