@@ -19,16 +19,19 @@ Freezer general utils functions
 import datetime
 import errno
 import fnmatch as fn
+import logging
 import os
 import subprocess
 import sys
 import time
 
 from distutils import spawn as distspawn
+from freezer.exceptions import utils
 from functools import wraps
 from oslo_log import log
 from six.moves import configparser
 
+logging.getLogger('botocore').setLevel(logging.WARNING)
 
 LOG = log.getLogger(__name__)
 
@@ -210,6 +213,36 @@ class Bunch(object):
         return self.__dict__.get(item)
 
 
+class S3ResponseStream(object):
+    """
+    Readable and iterable object body response wrapper.
+    """
+
+    def __init__(self, data, chunk_size):
+        """
+        Wrap the underlying response
+        :param data: the response to wrap
+        :param chunk_size: number of bytes to return each iteration/next call
+        """
+        self.data = data
+        self.chunk_size = chunk_size
+
+    def read(self, length=None):
+        return self.data.read(length)
+
+    def __iter__(self):
+        return self
+
+    def next(self):
+        buf = self.read(self.chunk_size)
+        if not buf:
+            raise StopIteration()
+        return buf
+
+    def __next__(self):
+        return self.next()
+
+
 class ReSizeStream(object):
     """
     Iterator/File-like object for changing size of chunk in stream
@@ -241,9 +274,11 @@ class ReSizeStream(object):
             stop = False
             while not stop and len(self.reminder) < chunk_size:
                 try:
-                    self.reminder += next(self.stream)
-                except TypeError:
-                    self.reminder += self.stream.next()
+                    next_method = getattr(self.stream, 'next', None)
+                    if callable(next_method):
+                        self.reminder += self.stream.next()
+                    else:
+                        self.reminder += next(self.stream)
                 except StopIteration:
                     stop = True
             if stop:
@@ -512,3 +547,14 @@ def set_max_process_priority():
         ])
     except Exception as priority_error:
         LOG.warning('Priority: {0}'.format(priority_error))
+
+
+def wait_for(condition_func, wait_interval, timeout, message=None, kwargs={}):
+    while timeout > 0:
+        if condition_func(**kwargs):
+            return
+
+        time.sleep(wait_interval)
+        timeout -= wait_interval
+
+    raise utils.TimeoutException(message)
